import { __decorate, __metadata } from 'tslib';
import { EventEmitter, Component, forwardRef, ChangeDetectionStrategy, Inject, ChangeDetectorRef, NgZone, Input, Output, NgModule } from '@angular/core';
import { DOCUMENT, CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { InputNumber } from '@ng-util/util';
import { NuLazyService } from '@ng-util/lazy';

class UEditorConfig {
}

const isSSR = !(typeof document === 'object' && !!document);
let _hook_finished = false;
class UEditorComponent {
    constructor(lazySrv, cog, doc, cd, zone) {
        this.lazySrv = lazySrv;
        this.cog = cog;
        this.doc = doc;
        this.cd = cd;
        this.zone = zone;
        this.inited = false;
        this.events = {};
        this.loading = true;
        this.id = `_ueditor-${Math.random().toString(36).substring(2)}`;
        this.loadingTip = '加载中...';
        this._disabled = false;
        this.delay = 50;
        this.onPreReady = new EventEmitter();
        this.onReady = new EventEmitter();
        this.onDestroy = new EventEmitter();
        this.onChange = () => { };
        this.onTouched = () => { };
    }
    set disabled(value) {
        this._disabled = value;
        this.setDisabled();
    }
    /**
     * 获取UE实例
     */
    get Instance() {
        return this.instance;
    }
    _getWin() {
        return this.doc.defaultView || window;
    }
    ngOnInit() {
        this.inited = true;
    }
    ngAfterViewInit() {
        if (isSSR) {
            return;
        }
        // 已经存在对象无须进入懒加载模式
        if (this._getWin().UE) {
            this.initDelay();
            return;
        }
        this.lazySrv.monitor(this.cog.js).subscribe(() => this.initDelay());
        this.lazySrv.load(this.cog.js);
    }
    ngOnChanges(changes) {
        if (this.inited && changes.config) {
            this.destroy();
            this.initDelay();
        }
    }
    initDelay() {
        setTimeout(() => this.init(), this.delay);
    }
    init() {
        const UE = this._getWin().UE;
        if (!UE) {
            throw new Error('uedito js文件加载失败');
        }
        if (this.instance) {
            return;
        }
        // registrer hook
        if (this.cog.hook && !_hook_finished) {
            _hook_finished = true;
            this.cog.hook(UE);
        }
        this.onPreReady.emit(this);
        const opt = Object.assign(Object.assign({}, this.cog.options), this.config);
        this.zone.runOutsideAngular(() => {
            const ueditor = UE.getEditor(this.id, opt);
            ueditor.ready(() => {
                this.instance = ueditor;
                if (this.value) {
                    this.instance.setContent(this.value);
                }
                this.onReady.emit(this);
            });
            ueditor.addListener('contentChange', () => {
                this.value = ueditor.getContent();
                this.zone.run(() => this.onChange(this.value));
            });
        });
        this.loading = false;
        this.cd.detectChanges();
    }
    destroy() {
        if (this.instance) {
            this.zone.runOutsideAngular(() => {
                Object.keys(this.events).forEach((name) => this.instance.removeListener(name, this.events[name]));
                this.instance.removeListener('ready');
                this.instance.removeListener('contentChange');
                // 由于此时 Angular 已经移除 DOM，可能会引起内部无法访问产生异常
                // https://github.com/cipchk/ngx-ueditor/issues/62
                try {
                    this.instance.destroy();
                    this.instance = null;
                }
                catch (_a) { }
            });
        }
        this.onDestroy.emit();
    }
    setDisabled() {
        if (!this.instance) {
            return;
        }
        if (this._disabled) {
            this.instance.setDisabled();
        }
        else {
            this.instance.setEnabled();
        }
    }
    /**
     * 设置编辑器语言
     */
    setLanguage(lang) {
        const UE = this._getWin().UE;
        return this.lazySrv
            .load(`${this.cog.options.UEDITOR_HOME_URL}/lang/${lang}/${lang}.js`)
            .then(() => {
            this.destroy();
            // 清空语言
            if (!UE._bak_I18N) {
                UE._bak_I18N = UE.I18N;
            }
            UE.I18N = {};
            UE.I18N[lang] = UE._bak_I18N[lang];
            this.initDelay();
        });
    }
    /**
     * 添加编辑器事件
     */
    addListener(eventName, fn) {
        if (this.events[eventName]) {
            return;
        }
        this.events[eventName] = fn;
        this.instance.addListener(eventName, fn);
    }
    /**
     * 移除编辑器事件
     */
    removeListener(eventName) {
        if (!this.events[eventName]) {
            return;
        }
        this.instance.removeListener(eventName, this.events[eventName]);
        delete this.events[eventName];
    }
    ngOnDestroy() {
        this.destroy();
    }
    writeValue(value) {
        this.value = value;
        if (this.instance) {
            this.instance.setContent(this.value);
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        this.setDisabled();
    }
}
UEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ueditor',
                template: `
    <textarea id="{{ id }}" class="ueditor-textarea"></textarea>
    <div *ngIf="loading" class="loading" [innerHTML]="loadingTip"></div>
  `,
                preserveWhitespaces: false,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => UEditorComponent),
                        multi: true,
                    },
                ],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`
      :host {
        line-height: initial;
      }
      :host .ueditor-textarea {
        display: none;
      }
    `]
            },] }
];
UEditorComponent.ctorParameters = () => [
    { type: NuLazyService },
    { type: UEditorConfig },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: ChangeDetectorRef },
    { type: NgZone }
];
UEditorComponent.propDecorators = {
    disabled: [{ type: Input }],
    config: [{ type: Input }],
    loadingTip: [{ type: Input }],
    delay: [{ type: Input }],
    onPreReady: [{ type: Output }],
    onReady: [{ type: Output }],
    onDestroy: [{ type: Output }]
};
__decorate([
    InputNumber(),
    __metadata("design:type", Object)
], UEditorComponent.prototype, "delay", void 0);

class UEditorModule {
    static forRoot(config) {
        return {
            ngModule: UEditorModule,
            providers: [{ provide: UEditorConfig, useValue: config }],
        };
    }
}
UEditorModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [UEditorComponent],
                exports: [UEditorComponent],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { UEditorComponent, UEditorConfig, UEditorModule };
//# sourceMappingURL=ngx-ueditor.js.map
